/*
  Macro           Test_RGB_color_support
  Author          Carlo Hogeveen
  Website         ecarlo.nl/tse
  Compatibility   TSE v4.50 rc 17 upwards, all variants.
  Version         v1.1   13 Jan 2024


  In Linux this tool lets us visually and safely test if the current Linux
  terminal or command line supports RGB colors.

  In Windows GUI TSE it always shows the three pretty screens that we hope to
  see in Linux.


  The tool is not dependent on TSE settings. No configuration is needed.

  Just compile and start the tool as a macro, and keep pressing the <Enter> key
  until the tool closes TSE.

  If between <Enter> keys you see three screens with smoothly blended colors,
  then the current Linux terminal or command line supports RGB colors,
  otherwise it does not.

  The screen might temporarily get mangled in some Linux terminals and command
  lines that do not support RGB colors.
  Just keep pressing <Enter>.
  The final <Enter> key will reset the Linux screen to remedy this.

  Screen resetting might not clear TSE's start-up panel, which is harmless.
  It might also overwrite the recent terminal screen's scroll history with TSE
  screens, which is an inconvenience, but the lesser one in my opinion.


  TODO
    MUST
    SHOULD
    COULD
    - Extend this to safely and user-friendly test of which TSE ANSI color
      driver best supports the current LINUX terminal or command line.
      But maybe that should be another tool, that also allows TSE to remember
      such a setting per Linux terminal/command line type.
    WONT


  HISTORY

  v1.0.1   5 Sep 2023
    Fixed: The shown color mixes did not match for two of the button names.

  v1.1    13 Jan 2024
    Updated the documentation and usage for Linux TSE v4.50 rc 17 upwards.
    Made it independent of TSE settings.
    Added resetting the terminal / command line screen afterwards.
    Perfected its existing functionality.

*/





/*
  T E C H N I C A L   B A C K G R O U N D   I N F O


  TEST NOTE
  I do have a Linux system with not-RGB-compatible command line that mangles
  the screen. The tool correctly resets the screen there.

*/





// Start compatibility checks

#ifndef INTERNAL_VERSION
  #define INTERNAL_VERSION 0
#endif

#if INTERNAL_VERSION < 12379



  This tool requires at least TSE v4.50 rc 17.



#endif

// End compatibility checks.




#ifdef LINUX
  proc reset_linux_screen()
    Dos('printf "\ec"', _DONT_CLEAR_|_DONT_PROMPT_)
  end reset_linux_screen
#endif

proc test_rgb_color_support()
  integer color_mix           = 0
  string  mixed_message  [14] = ''
  string  msg1           [80] = ''
  string  msg2           [80] = ''
  integer old_blue_color      = GetColorTableValue(_BACKGROUND_, _BLUE_)
  integer old_cyan_color      = GetColorTableValue(_BACKGROUND_, _CYAN_)
  integer rgb_color           = 0
  integer rgb_color_range     = 255 * 255 * 255
  integer x                   = 0
  integer x_step              = 0
  integer y                   = 0
  integer y_step              = 0

  for color_mix = 1 to 3
    case color_mix
      when 1
        mixed_message = 'RED and GREEN'
        x_step = (255 / Query(ScreenCols)) * 256 * 256
        y_step = (255 / Query(ScreenRows)) * 256
      when 2
        mixed_message = 'GREEN and BLUE'
        x_step = (255 / Query(ScreenCols)) * 256
        y_step = (255 / Query(ScreenRows))
      otherwise
        mixed_message = 'BLUE and RED'
        x_step = (255 / Query(ScreenCols))
        y_step = (255 / Query(ScreenRows)) * 256 * 256
    endcase

    msg1 = Format(' If RGB works, then the whole screen should now be colored, ')
    msg2 = Format(' with ', mixed_message,
                  ' in the corners, blending in the middle.')

    PopWinOpen(1, 1, Query(ScreenCols), Query(ScreenRows), 0, '',
               Color(Black ON Black))

    // TSE optimizes PutStrAttrXY() by not rewriting existing screen attributes.
    // It is extremely important to understand, that this applies to the color
    // attribute, not to the color of the color attribute.
    // For ordinary text that is great, but it hampers us just recoloring text,
    // because the color attribute we use might already occur on the screen.
    // So we first recolor the screen with a base color attribute, here blue.
    // The color attribute cannot be black: PutStrAttrXY() has a problem with
    // it.
    SetColorTableValue(_BACKGROUND_, _BLUE_, 0)
    for y = 1 to Query(ScreenRows)
      for x = 1 to Query(ScreenCols)
        PutStrAttrXY(x, y, ' ', '', Color(Blue ON Blue))
      endfor
    endfor

    // Now color the screen with another color atttribute, here cyan.
    for y = 1 to Query(ScreenRows)
      for x = 1 to Query(ScreenCols)
        rgb_color = (x * x_step + y * y_step) mod rgb_color_range
        SetColorTableValue(_BACKGROUND_, _CYAN_, rgb_color)
        PutStrAttrXY(x, y, ' ', '', Color(Cyan ON Cyan))
      endfor
    endfor

    SetColorTableValue(_BACKGROUND_, _BLUE_, old_blue_color )
    SetColorTableValue(_BACKGROUND_, _CYAN_, old_cyan_color)
    PutStrAttrXY(10, 10, Format(msg1: -60)                , '', Query(TextAttr))
    PutStrAttrXY(10, 11, Format(msg2: -60)                , '', Query(TextAttr))
    PutStrAttrXY(10, 15, ' Press any key to continue ... ', '', Query(TextAttr))
    GetKey()
    PopWinClose()
  endfor
end test_rgb_color_support


proc Main()
  #ifdef LINUX
    if Query(CurrAnsiColor) <> 1
      Warn("TSE's"; 'menu option "ANSI Color driver" is temporarily set to "RGB".')
      Set(CurrAnsiColor, 1)
    endif
    test_rgb_color_support()
    reset_linux_screen()
    Set(QuitToPrompt, OFF)
    PushKey(<x>)  // Skip menu.
    PurgeMacro(CurrMacroFilename())
    AbandonEditor()
  #else
    if isGUI()
      test_rgb_color_support()
    else
      Warn('The Console version of Windows TSE (e32.exe) does not support RGB colors.')
    endif
    PurgeMacro(CurrMacroFilename())
  #endif
end Main

